// 3D Interaction Project by Song Gao, Department of Geography, UC Santa Barbara
// MAT 259, 2014 Winter Quarter

//import gifAnimation.*; // http://www.extrapixel.ch/processing/gifAnimation/
import peasy.*; // http://mrfeinberg.com/peasycam/

PeasyCam cam;
//GifMaker gifExport;

//Global Variables
String dcCoordsFilename= "MDS_coordinates.csv";
float[][] dcCoords=new float[999][2]; // the 2D coordinates for the dewey classes
String dcTemporalFilename= "2006_2013_MonthlyDeweyCheckouts_new.csv";
float[][][] dcMonthlyData=new float[999][8][12]; //Dewey Classes Monthly Checkouts for the year 2006 to 2013
int globeSize=15; // to control the largest size of each globe
float prob=0.066; // the probability threshold
float longi=0.0;
float lati=0.0;
PFont gFont1 = null;
int minFontSize=5; // the minimum of the font size for dewey class number
boolean starFlag= true;    // the boolean flag for stars
boolean dcLabelFlag= false; // the boolean flag for dewey class labels
boolean legendFlag= false; // the boolean flag for the color legend

// A Polar coordinate
float r = 0;
float theta = 0;
float scale = 5;
float pi=4*atan(1);
int stars=100000; // the number of simulatd stars
int Rmax=500; // galaxy radius
float speed=0.02;  // rotation speed 
// stars follow elliptic orbits around the center
float eratio=0.85*1.2*1.2; // ellipse ratio
float etwist=8.0/Rmax; // twisting factor (orbit axes depend on radius)
float []angle=new float[stars];
float []radius=new float[stars];
float cx; float cy; //galaxy center
color col=color(255,255,192); // sparkling yellow stars
 
void setup(){
  size(1000, 800,P3D);
  background(0); // back to black
  speed=speed/frameRate;
  cam = new PeasyCam(this,0, 0, 0, 350);
  cam.setMinimumDistance(50);
  cam.setMaximumDistance(600);
  cam.lookAt(width/4, height/4, 100);
    
  //Load Saved Data from local files
    //coordinates
    String [] textRows = loadStrings(dcCoordsFilename); 
    for (int row=1; row<textRows.length; row++) {
      String textData[] = split(textRows[row], ","); // split the data in each line from comma
      //println("ID: "+textData[0]+" x: "+textData[1]+" y: "+textData[2]);
      dcCoords[PApplet.parseInt(textData[0])][0]=PApplet.parseFloat(textData[1]);
      dcCoords[PApplet.parseInt(textData[0])][1]=PApplet.parseFloat(textData[2]);
      //println("ID: "+textData[0]+" x: "+dcCoords[row-1][0]+" y: "+dcCoords[row-1][1]) ;
    }
     //temporal checkouts data
    String [] textTemporalRows = loadStrings(dcTemporalFilename); 
    for (int row=1; row<textTemporalRows.length; row++) {
      String textTData[] = split(textTemporalRows[row], ","); // split the data in each line from comma
      dcMonthlyData[PApplet.parseInt(textTData[2])][PApplet.parseInt(textTData[0])-2006][PApplet.parseInt(textTData[1])-1]=PApplet.parseFloat(textTData[4]);
      //println(textTData[0]+", "+textTData[1]+", "+textTData[2]+", "+textTData[4]);
  }
     //gifExport = new gifAnimation.GifMaker(this, "galaxy.gif");
     //gifExport.setRepeat(0);  
}

void draw(){
  background(0); // back to black
  stroke(col);
  //rotateX(-45);
  
  //drawing the sparkling stars
   if(starFlag){
   drawingStars();
    }
   
     //drawing the dewey classes
     noStroke();
     lights();
     for (int dcID=0;dcID<999;dcID++){   
       pushMatrix();
       longi=dcCoords[dcID][0]*width/2;
       lati=dcCoords[dcID][1]*height/2;
       print ("ID:"+dcID+" longi: "+longi+" lati: "+lati+"\n");
       translate(longi+10,lati+10,0);
      
       //Dewey Class ID Labels
    if(dcMonthlyData[dcID][0][0]>0.3 || dcID==394){
       if(dcLabelFlag){
         colorMode(RGB,255,255,255);    
         fill(255,255,255);
         float labelSize=minFontSize+dcMonthlyData[dcID][0][0]*25;
         if(labelSize<35){
           gFont1 = createFont("Arial", labelSize, false);
         } 
         else{gFont1 = createFont("Arial", 25, false);}       
         textFont(gFont1); 
         String str_dcID=Integer.toString(dcID);
         text(str_dcID, 0, 0, 20);
         }
       }
       // Dewey Classes Temporal Checkouts    
       for (int r=0;r<8;r=r+1){
       for(int theta=0;theta<12;theta++) {
          float x = r * cos(theta*30*PI/180)*scale;
          float y = r * sin(theta*30*PI/180)*scale;
          translate(x,y,-r);
          shininess(2.0);
          colorMode(HSB,1000,100,100);    
          fill(dcID,100,95);
          float dcGSize=dcMonthlyData[dcID][r][theta]*globeSize; 
          //print ("ID:"+dcID+" longi: "+longi+" lati: "+lati+" size: "+dcGSize+"\n");
          
          if(dcGSize>prob*globeSize && dcGSize < globeSize){
          sphere(dcGSize);
          }
          else if(dcGSize >= globeSize) { sphere(globeSize);}
          else {point(0,0,0);}
        }
     }
     popMatrix();
}
     //draw color legend
     if(legendFlag){drawColorLegend();}

//gifExport.setDelay(1); gifExport.addFrame();
save("3D.png");
}
 
void keyPressed() {
  if(keyCode == UP) { eratio=eratio*1.02; }
  if(keyCode == DOWN) { eratio=eratio/1.02; }
  if(keyCode == LEFT) { etwist=etwist+0.001; }
  if(keyCode == RIGHT) { etwist=etwist-0.001; }
  if(keyCode == 'S'){ 
    if(starFlag){starFlag=false;}
    else{starFlag=true;}
  }
  if(keyCode == 'T'){ 
    if(dcLabelFlag){dcLabelFlag=false;}
    else{dcLabelFlag=true;}
  }
  if(keyCode == 'L'){ 
    if(legendFlag){legendFlag=false;}
    else{legendFlag=true;}
  }
  
}

//void keyReleased() {
//  if(keyCode == 'S'){ starFlag=true;}
//}

//drawing the sparkling stars
void drawingStars(){
  // the location of "center"
  cx = width/4;
  cy = height/4;
  pushMatrix();
  translate(0,0,-300);
  // itit stars
  for (int i=0; i< stars; i++){
    angle[i]= random(0,2*pi);
    radius[i]=random(1,Rmax);
  } 
  for (int i =0; i< stars; i++){
    float r=radius[i];
    float a=angle[i]+speed; // increment angle
    angle[i]=a;
    float x = r*sin(a);
    float y= r*eratio*cos(a);
    float b=r*etwist;
    float s=sin(b);
    float c=cos(b);
    point(cx+s*x+c*y,cy+c*x-s*y,s);
  }
   popMatrix();
}

//drawing the colorbars legend
void drawColorLegend(){
 translate(-30,-30,20);
 textAlign(LEFT, CENTER);
 gFont1 = createFont("Arial", 20, false);
 textFont(gFont1);
 fill(#FFFFFF);
 text("Dewey Classes Galaxy",300,0,10);
  
  colorMode(HSB,1000,100,100);    
  for (int i=0;i<10;i++){
       for(int j=0;j<100;j=j+1){
          pushMatrix();
          translate(j*2,i*height/70,0);
          fill(i*100+j,100,95);
          sphere(1);
          if(j==0){
            gFont1 = createFont("Arial", 10, false);    
            textFont(gFont1); 
            String str_dcID=Integer.toString(i*100+j);
            //println(str_dcID);
            text(str_dcID, -12, 1, 10);
          }
           if(j==99){
            gFont1 = createFont("Arial", 10, false);    
            textFont(gFont1); 
            String str_dcID=Integer.toString(i*100+j);
            //println(str_dcID);
            text(str_dcID, 3, 1, 10);
          }
          popMatrix();
  }
 }//ending loop
  
//  gFont1 = createFont("Arial", 10, false);
//  textFont(gFont1);
//  fill(#FFFFFF);
//  textAlign(RIGHT, CENTER);
//  text("by Song Gao, 2014 MAT259",600,10,10);
  
}
 
